/*******************************************************************************

        Shows how the basic functionality of Logger operates.

*******************************************************************************/

private import std.string;

private import mango.log.Logger,
               mango.log.Configurator;

/*******************************************************************************

*******************************************************************************/

private class Sieve
{
        private Logger logger;
        
        /***********************************************************************

                Initialize the Sieve class 

        ***********************************************************************/

        this()
        {
                // get a logger for this object. Could make this static instead
                logger = Logger.getLogger ("example.logging.Sieve");
        }

        /***********************************************************************

                Search for a set of prime numbers

        ***********************************************************************/

        void compute (uint max)
        {
                byte*   feld;
                uint    teste=1,
                        mom,
                        hits=0,
                        count, 
                        s=0,
                        e = 1;

                void set (byte* f, uint x)
                {
                        *(f+(x)/16) |=1 <<(((x)%16)/2);
                }

                byte test (byte* f, uint x)
                {
                        return (*(f+(x)/16) & (1<<(((x)%16)/2)));
                }

                // information level
                logger.info ("Searching prime numbers to : " ~ std.string.toString(max));

                feld = new byte[max / 16 + 1];

                // get milliseconds since application began
                long begin = logger.getUptime();

                while ((teste += 2) < max)
                        if (! test (feld, teste)) 
                           {
                           if  ((++hits & 0x0f) == 0) 
                                // more information level
                                logger.info ("found " ~ std.string.toString(hits)); 

                           for (mom=3*teste; mom < max; mom += teste<<1) 
                                set (feld, mom);
                           }

                // get number of milliseconds we took to compute
                long period = logger.getUptime() - begin;

                if (hits)
                    // more information
                    logger.info (std.string.toString(hits) ~ " prime numbers found in "~ std.string.toString(period) ~ " millisecs");
                else
                   // a warning level
                   logger.warn ("no prime numbers found");
        
                // check to see if we're enabled for 
                // tracing before we expend a lot of effort
                if (logger.isEnabled (logger.Level.Trace))
                   {        
                   e = max;
                   count = 0 - 2; 
                   if (s % 2 == 0) 
                       count++;
           
                   while ((count+=2) < e) 
                           if (! test (feld, count)) 
                               // log trace information
                               logger.trace ("prime found: " ~ std.string.toString(count));
                   }
        }
}


/*******************************************************************************

        Create a Sieve and have it compute a bunch of prime numbers.

*******************************************************************************/

void main()
{
        // get a logger to reresent this module. We could just as
        // easily share a name with some other module(s)
        Logger logger = Logger.getLogger ("example.logging");
        
        // set up a basic logging configuration
        BasicConfigurator.configure();

        try {
            Sieve sieve = new Sieve;

            sieve.compute (1000);

            } catch (Exception x)
                    {
                    // log the exception as an error
                    logger.error ("Exception: " ~ x.toString);
                    }
}
