/*******************************************************************************

        @file ICU.d
        
        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, October 2004      
        @author         Kris


	Note that this package and documentation is built around the ICU 
	project (http://oss.software.ibm.com/icu/). Below is the license 
	statement as specified by that software:


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


	ICU License - ICU 1.8.1 and later

	COPYRIGHT AND PERMISSION NOTICE

	Copyright (c) 1995-2003 International Business Machines Corporation and 
	others.

	All rights reserved.

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, and/or sell copies of the Software, and to permit persons
	to whom the Software is furnished to do so, provided that the above
	copyright notice(s) and this permission notice appear in all copies of
	the Software and that both the above copyright notice(s) and this
	permission notice appear in supporting documentation.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
	OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
	HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
	INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
	FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
	WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

	Except as contained in this notice, the name of a copyright holder
	shall not be used in advertising or otherwise to promote the sale, use
	or other dealings in this Software without prior written authorization
	of the copyright holder.

	----------------------------------------------------------------------

	All trademarks and registered trademarks mentioned herein are the 
	property of their respective owners.

*******************************************************************************/

module mango.icu.ICU;

/*******************************************************************************

*******************************************************************************/

private static extern (C) uint strlen (char *s);
private static extern (C) uint wcslen (wchar *s);

/*******************************************************************************
	
	Some low-level routines to help bind the ICU C-API to D.

        See http://oss.software.ibm.com/icu/ for full documentation on the
	ICU package

*******************************************************************************/

protected class ICU
{
        /***********************************************************************
        
                Use this for the primary argument-type to most ICU functions

        ***********************************************************************/

        protected typedef void* Handle;

        /***********************************************************************
        
                ICU error codes (the ones which are referenced)

        ***********************************************************************/

        protected enum  Error:int 
                        {
                        OK, 
                        BufferOverflow=15
                        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final bool isError (Error e)
        {
                return e > 0;
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final void exception (char[] msg)
        {
                throw new ICUException (msg);
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final void testError (Error e, char[] msg)
        {
                if (e > 0)
                    exception (msg);
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final char* toString (char[] string)
        {
                if (! string.length)
                      return ((cast(char*) string) ? "" : cast(char*) null);

                if (* (&string[0] + string.length))
                   {
                   // Need to make a copy
                   char[] copy = new char [string.length + 1];
                   copy [0..string.length] = string;
                   copy [string.length] = 0;
                   string = copy;
                   }
                return string;
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final wchar* toString (wchar[] string)
        {
                return cast(wchar*) toString (cast(char[]) string);
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final uint length (char* s)
        {
                return strlen (s);
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final uint length (wchar* s)
        {
                return wcslen (s);
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final char[] toArray (char* s)
        {
                if (s)
                    return s[0..strlen (s)];
                return null;
        }

        /***********************************************************************
        
        ***********************************************************************/

        protected static final wchar[] toArray (wchar* s)
        {
                if (s)
                    return s[0..wcslen (s)];
                return null;   
        }
}


/*******************************************************************************

*******************************************************************************/

class ICUException : Exception
{
        /***********************************************************************
        
                Construct exception with the provided text string

        ***********************************************************************/

        this (char[] msg)
        {
                super (msg);
        }
}


/*******************************************************************************

*******************************************************************************/

class FunctionLoader
{
        version (Win32)
        {
                private import std.c.windows.windows;

                /***************************************************************

                ***************************************************************/

                protected struct Bind
                {
                        void**  fnc;
                        char[]  name;      
                }

                /***************************************************************

                ***************************************************************/

                static final void* bind (char[] library, inout Bind[] targets)
                {
                        static char[] vrsn = "_3_0\0";

                        HANDLE lib = LoadLibraryA (ICU.toString(library));

                        foreach (Bind b; targets)
                                {
                                char[] name = b.name ~ vrsn;
                                *b.fnc = GetProcAddress (lib, name);
                                if (*b.fnc)
                                    {}//printf ("bound '%.*s'\n", name);
                                else
                                   throw new Error ("required " ~ name ~ " in library " ~ library);
                                }
                        return lib;
                }

                /***************************************************************

                ***************************************************************/

                static final void unbind (void* library)
                {       
                        //FreeLibrary (cast(HANDLE) library);
                }
        }


        /***********************************************************************
        
        ***********************************************************************/

        version (linux)
        {
                static this ()
                {
                        throw new Exception ("ICU is currently unsupported on linux");
                }
        }
}


