/*******************************************************************************

        @file UChar.d
        
        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, October 2004      
        @author         Kris
        

	Note that this package and documentation is built around the ICU 
	project (http://oss.software.ibm.com/icu/). Below is the license 
	statement as specified by that software:


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


	ICU License - ICU 1.8.1 and later

	COPYRIGHT AND PERMISSION NOTICE

	Copyright (c) 1995-2003 International Business Machines Corporation and 
	others.

	All rights reserved.

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, and/or sell copies of the Software, and to permit persons
	to whom the Software is furnished to do so, provided that the above
	copyright notice(s) and this permission notice appear in all copies of
	the Software and that both the above copyright notice(s) and this
	permission notice appear in supporting documentation.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
	OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
	HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
	INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
	FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
	WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

	Except as contained in this notice, the name of a copyright holder
	shall not be used in advertising or otherwise to promote the sale, use
	or other dealings in this Software without prior written authorization
	of the copyright holder.

	----------------------------------------------------------------------

	All trademarks and registered trademarks mentioned herein are the 
	property of their respective owners.

*******************************************************************************/

module mango.icu.UChar;

private import  mango.icu.ICU;

/*******************************************************************************

        This API provides low-level access to the Unicode Character 
        Database. In addition to raw property values, some convenience 
        functions calculate derived properties, for example for Java-style 
        programming.

        Unicode assigns each code point (not just assigned character) 
        values for many properties. Most of them are simple boolean 
        flags, or constants from a small enumerated list. For some 
        properties, values are strings or other relatively more complex 
        types.

        For more information see "About the Unicode Character Database" 
        (http://www.unicode.org/ucd/) and the ICU User Guide chapter on 
        Properties (http://oss.software.ibm.com/icu/userguide/properties.html).

        Many functions are designed to match java.lang.Character functions. 
        See the individual function documentation, and see the JDK 1.4.1 
        java.lang.Character documentation at 
        http://java.sun.com/j2se/1.4.1/docs/api/java/lang/Character.html

        There are also functions that provide easy migration from C/POSIX 
        functions like isblank(). Their use is generally discouraged because 
        the C/POSIX standards do not define their semantics beyond the ASCII 
        range, which means that different implementations exhibit very different 
        behavior. Instead, Unicode properties should be used directly.

        There are also only a few, broad C/POSIX character classes, and they 
        tend to be used for conflicting purposes. For example, the "isalpha()" 
        class is sometimes used to determine word boundaries, while a more 
        sophisticated approach would at least distinguish initial letters from 
        continuation characters (the latter including combining marks). (In 
        ICU, BreakIterator is the most sophisticated API for word boundaries.) 
        Another example: There is no "istitle()" class for titlecase characters.

        A summary of the behavior of some C/POSIX character classification 
        implementations for Unicode is available at 
        http://oss.software.ibm.com/cvs/icu/~checkout~/icuhtml/design/posix_classes.html

        See <A HREF="http://oss.software.ibm.com/icu/apiref/uchar_8h.html">
        this page</A> for full details.

*******************************************************************************/

class UChar : ICU
{
        /***********************************************************************
        
        ***********************************************************************/

        public static extern (C) 
        {
        /***********************************************************************
        
                Check if a code point has the Alphabetic Unicode property.

        ***********************************************************************/

        bool function (dchar c) isUAlphabetic;

        /***********************************************************************
        
                Check if a code point has the Lowercase Unicode property.

        ***********************************************************************/

        bool function (dchar c) isULowercase;

        /***********************************************************************
        
                Check if a code point has the Uppercase Unicode property.

        ***********************************************************************/

        bool function (dchar c) isUUppercase;

        /***********************************************************************
        
                Check if a code point has the White_Space Unicode property.

        ***********************************************************************/

        bool function (dchar c) isUWhiteSpace;

        /***********************************************************************
        
                Determines whether the specified code point has the general 
                category "Ll" (lowercase letter).

        ***********************************************************************/

        bool function (dchar c) isLower;

        /***********************************************************************
        
                Determines whether the specified code point has the general 
                category "Lu" (uppercase letter).

        ***********************************************************************/

        bool function (dchar c) isUpper;

        /***********************************************************************
        
                Determines whether the specified code point is a titlecase 
                letter.

        ***********************************************************************/

        bool function (dchar c) isTitle;

        /***********************************************************************
        
                Determines whether the specified code point is a digit 
                character according to Java.

        ***********************************************************************/

        bool function (dchar c) isDigit;

        /***********************************************************************
        
                Determines whether the specified code point is a letter 
                character.

        ***********************************************************************/

        bool function (dchar c) isAlpha;

        /***********************************************************************
        
                Determines whether the specified code point is an 
                alphanumeric character (letter or digit) according 
                to Java.

        ***********************************************************************/

        bool function (dchar c) isAlphaNumeric;

        /***********************************************************************
        
                Determines whether the specified code point is a 
                hexadecimal digit.

        ***********************************************************************/

        bool function (dchar c) isHexDigit;

        /***********************************************************************
        
                Determines whether the specified code point is a 
                punctuation character.

        ***********************************************************************/

        bool function (dchar c) isPunct;

        /***********************************************************************
        
                Determines whether the specified code point is a "graphic" 
                character (printable, excluding spaces).

        ***********************************************************************/

        bool function (dchar c) isGraph;

        /***********************************************************************
        
                Determines whether the specified code point is a "blank" 
                or "horizontal space", a character that visibly separates 
                words on a line.

        ***********************************************************************/

        bool function (dchar c) isBlank;

        /***********************************************************************
        
                Determines whether the specified code point is "defined", 
                which usually means that it is assigned a character.

        ***********************************************************************/

        bool function (dchar c) isDefined;

        /***********************************************************************
        
                Determines if the specified character is a space character
                 or not.

        ***********************************************************************/

        bool function (dchar c) isSpace;

        /***********************************************************************
        
                Determine if the specified code point is a space character 
                according to Java.

        ***********************************************************************/

        bool function (dchar c) isJavaSpaceChar;

        /***********************************************************************
        
                Determines if the specified code point is a whitespace 
                character according to Java/ICU.

        ***********************************************************************/

        bool function (dchar c) isWhiteSpace;

        /***********************************************************************
        
                Determines whether the specified code point is a control 
                character (as defined by this function).

        ***********************************************************************/

        bool function (dchar c) isCtrl;

        /***********************************************************************
        
                Determines whether the specified code point is an ISO 
                control code.

        ***********************************************************************/

        bool function (dchar c) isISOControl;

        /***********************************************************************
        
                Determines whether the specified code point is a 
                printable character.

        ***********************************************************************/

        bool function (dchar c) isPrint;

        /***********************************************************************
        
                Determines whether the specified code point is a 
                base character.

        ***********************************************************************/

        bool function (dchar c) isBase;

        /***********************************************************************
        
                Determines if the specified character is permissible 
                as the first character in an identifier according to 
                Unicode (The Unicode Standard, Version 3.0, chapter 
                5.16 Identifiers).

        ***********************************************************************/

        bool function (dchar c) isIDStart;

        /***********************************************************************
        
                Determines if the specified character is permissible 
                in an identifier according to Java.

        ***********************************************************************/

        bool function (dchar c) isIDPart;

        /***********************************************************************
        
                Determines if the specified character should be regarded
                as an ignorable character in an identifier, according 
                to Java.

        ***********************************************************************/

        bool function (dchar c) isIDIgnorable;

        /***********************************************************************
        
                Determines if the specified character is permissible 
                as the first character in a Java identifier.

        ***********************************************************************/

        bool function (dchar c) isJavaIDStart;

        /***********************************************************************
        
                Determines if the specified character is permissible 
                in a Java identifier.

        ***********************************************************************/

        bool function (dchar c) isJavaIDPart;

        /***********************************************************************
        
                Determines whether the code point has the Bidi_Mirrored 
                property.

        ***********************************************************************/

        bool function (dchar c) isMirrored;

        /***********************************************************************
        
                Returns the decimal digit value of a decimal digit character.

        ***********************************************************************/

        ubyte function (dchar c) charDigitValue;

        /***********************************************************************
        
                Maps the specified character to a "mirror-image" character.

        ***********************************************************************/

        dchar function (dchar c) charMirror;

        /***********************************************************************
        
                Returns the general category value for the code point.

        ***********************************************************************/

        ubyte function (dchar c) charType;

        /***********************************************************************
        
                Returns the combining class of the code point as specified 
                in UnicodeData.txt.

        ***********************************************************************/

        ubyte function (dchar c) getCombiningClass;

        /***********************************************************************
        
                The given character is mapped to its lowercase equivalent 
                according to UnicodeData.txt; if the character has no 
                lowercase equivalent, the character itself is returned.

        ***********************************************************************/

        dchar function (dchar c) toLower;

        /***********************************************************************
        
                The given character is mapped to its uppercase equivalent 
                according to UnicodeData.txt; if the character has no 
                uppercase equivalent, the character itself is returned.

        ***********************************************************************/

        dchar function (dchar c) toUpper;

        /***********************************************************************
        
                The given character is mapped to its titlecase equivalent 
                according to UnicodeData.txt; if none is defined, the 
                character itself is returned.

        ***********************************************************************/

        dchar function (dchar c) toTitle;

        /***********************************************************************
        
                The given character is mapped to its case folding equivalent 
                according to UnicodeData.txt and CaseFolding.txt; if the 
                character has no case folding equivalent, the character 
                itself is returned.

        ***********************************************************************/

        dchar function (dchar c, uint options) foldCase;

        /***********************************************************************
        
                Returns the decimal digit value of the code point in the 
                specified radix.

        ***********************************************************************/

        uint function (dchar ch, ubyte radix) digit;

        /***********************************************************************
        
                Determines the character representation for a specific 
                digit in the specified radix.

        ***********************************************************************/

        dchar function (uint digit, ubyte radix) forDigit;

        /***********************************************************************
        
                Get the numeric value for a Unicode code point as defined 
                in the Unicode Character Database.

        ***********************************************************************/

        double function (dchar c) getNumericValue;
        }
        

        /***********************************************************************
        
                Bind the ICU functions from a shared library. This is
                complicated by the issues regarding D and DLLs on the
                Windows platform

        ***********************************************************************/

        version (Win32)
        {
                private static void*    library;
                private static char[]   libraryName = "icuuc30.dll";     

                /***************************************************************

                ***************************************************************/

                static  FunctionLoader.Bind[] targets = 
                        [
                        {cast(void**) &forDigit,          "u_forDigit"},
                        {cast(void**) &digit,             "u_digit"},
                        {cast(void**) &foldCase,          "u_foldCase"},
                        {cast(void**) &toTitle,           "u_totitle"},
                        {cast(void**) &toUpper,           "u_toupper"},
                        {cast(void**) &toLower,           "u_tolower"},
                        {cast(void**) &charDigitValue,    "u_charDigitValue"},
                        {cast(void**) &getCombiningClass, "u_getCombiningClass"},
                        {cast(void**) &charType,          "u_charType"},
                        {cast(void**) &charMirror,        "u_charMirror"},
                        {cast(void**) &isJavaIDPart,      "u_isJavaIDPart"},
                        {cast(void**) &isJavaIDStart,     "u_isJavaIDStart"},
                        {cast(void**) &isIDIgnorable,     "u_isIDIgnorable"},
                        {cast(void**) &isIDPart,          "u_isIDPart"},
                        {cast(void**) &isIDStart,         "u_isIDStart"},
                        {cast(void**) &isMirrored,        "u_isMirrored"},
                        {cast(void**) &isBase,            "u_isbase"},
                        {cast(void**) &isPrint,           "u_isprint"},
                        {cast(void**) &isISOControl,      "u_isISOControl"},
                        {cast(void**) &isCtrl,            "u_iscntrl"},
                        {cast(void**) &isWhiteSpace,      "u_isWhitespace"},
                        {cast(void**) &isJavaSpaceChar,   "u_isJavaSpaceChar"},
                        {cast(void**) &isSpace,           "u_isspace"},
                        {cast(void**) &isDefined,         "u_isdefined"},
                        {cast(void**) &isBlank,           "u_isblank"},
                        {cast(void**) &isGraph,           "u_isgraph"},
                        {cast(void**) &isPunct,           "u_ispunct"},
                        {cast(void**) &isHexDigit,        "u_isxdigit"},
                        {cast(void**) &isAlpha,           "u_isalpha"},
                        {cast(void**) &isAlphaNumeric,    "u_isalnum"},
                        {cast(void**) &isDigit,           "u_isdigit"},
                        {cast(void**) &isTitle,           "u_istitle"},
                        {cast(void**) &isUpper,           "u_isupper"},
                        {cast(void**) &isLower,           "u_islower"},
                        {cast(void**) &isUAlphabetic,     "u_isUAlphabetic"},
                        {cast(void**) &isUWhiteSpace,     "u_isUWhiteSpace"},
                        {cast(void**) &isUUppercase,      "u_isUUppercase"},
                        {cast(void**) &isULowercase,      "u_isULowercase"},
                        {cast(void**) &getNumericValue,   "u_getNumericValue"},
                        ];

                /***************************************************************

                ***************************************************************/

                static this ()
                {
                        library = FunctionLoader.bind (libraryName, targets);
                }

                /***************************************************************

                ***************************************************************/

                static ~this ()
                {
                        FunctionLoader.unbind (library);
                }
        }
}
