/*******************************************************************************

        @file UTimeZone.d
        
        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, November 2004      
        @author         Kris

	Note that this package and documentation is built around the ICU 
	project (http://oss.software.ibm.com/icu/). Below is the license 
	statement as specified by that software:


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


	ICU License - ICU 1.8.1 and later

	COPYRIGHT AND PERMISSION NOTICE

	Copyright (c) 1995-2003 International Business Machines Corporation and 
	others.

	All rights reserved.

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, and/or sell copies of the Software, and to permit persons
	to whom the Software is furnished to do so, provided that the above
	copyright notice(s) and this permission notice appear in all copies of
	the Software and that both the above copyright notice(s) and this
	permission notice appear in supporting documentation.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
	OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
	HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
	INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
	FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
	NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
	WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

	Except as contained in this notice, the name of a copyright holder
	shall not be used in advertising or otherwise to promote the sale, use
	or other dealings in this Software without prior written authorization
	of the copyright holder.

	----------------------------------------------------------------------

	All trademarks and registered trademarks mentioned herein are the 
	property of their respective owners.

*******************************************************************************/

module mango.icu.UTimeZone;

private import  mango.icu.ICU,
                mango.icu.UString;

/*******************************************************************************

        A representation of a TimeZone. Unfortunately, ICU does not expose
        this as a seperate entity from the C-API, so we have to make do 
        with an approximation instead.

*******************************************************************************/

struct UTimeZone 
{
        public wchar[]  name;

        public static UTimeZone Default = {null};

        /***********************************************************************
        
                Get the default time zone.

        ***********************************************************************/

        static void getDefault (inout UTimeZone zone)
        {       
                uint format (wchar* dst, uint length, inout ICU.Error e)
                {
                        return ucal_getDefaultTimeZone (dst, length, e);
                }

                UString s = new UString(64);
                s.format (&format, "failed to get default time zone");
                zone.name = s.get();
        }

        /***********************************************************************
        
                Set the default time zone.

        ***********************************************************************/

        static void setDefault (inout UTimeZone zone)
        {       
                ICU.Error e;

                ucal_setDefaultTimeZone (ICU.toString (zone.name), e);
                ICU.testError (e, "failed to set default time zone");                
        }

        /***********************************************************************
        
                Return the amount of time in milliseconds that the clock 
                is advanced during daylight savings time for the given 
                time zone, or zero if the time zone does not observe daylight 
                savings time

        ***********************************************************************/

        static uint getDSTSavings (inout UTimeZone zone)
        {       
                ICU.Error e;

                uint x = ucal_getDSTSavings (ICU.toString (zone.name), e);
                ICU.testError (e, "failed to get DST savings");                
                return x;
        }


        /***********************************************************************
        
                Bind the ICU functions from a shared library. This is
                complicated by the issues regarding D and DLLs on the
                Windows platform

        ***********************************************************************/

        version (Win32)
        {
                private static void*    library;
                private static char[]   libraryName = "icuin30.dll";     

                /***************************************************************

                ***************************************************************/

                private static extern (C) 
                {
                        uint function (wchar*, uint, inout ICU.Error) ucal_getDefaultTimeZone;
                        void function (wchar*, inout ICU.Error) ucal_setDefaultTimeZone;
                        uint function (wchar*, inout ICU.Error) ucal_getDSTSavings;
                }

                /***************************************************************

                ***************************************************************/

                static  FunctionLoader.Bind[] targets = 
                        [
                        {cast(void**) &ucal_getDefaultTimeZone, "ucal_getDefaultTimeZone"}, 
                        {cast(void**) &ucal_setDefaultTimeZone, "ucal_setDefaultTimeZone"}, 
                        {cast(void**) &ucal_getDSTSavings,      "ucal_getDSTSavings"}, 
                        ];

                /***************************************************************

                ***************************************************************/

                static this ()
                {
                        library = FunctionLoader.bind (libraryName, targets);
                }

                /***************************************************************

                ***************************************************************/

                static ~this ()
                {
                        FunctionLoader.unbind (library);
                }
        }
}
