/*******************************************************************************

        @file EndianReader.d
        
        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, March 2004      
        @author         Kris


*******************************************************************************/

module mango.io.EndianReader;

public  import  mango.io.Reader;

private import  mango.base.ByteSwap;

/*******************************************************************************

        A pair of classes to handle byte-swapping for integral types. The
        basic EndianReader is transparent, whilst EndianReaderSwap is not.
        
        Use the static create() method to construct the appropriate reader.

*******************************************************************************/

class EndianReader : Reader
{       
        /***********************************************************************
        
                Construct EndianReader upon the given IBuffer. 

        ***********************************************************************/

        this (IBuffer buffer)
        {
                super (buffer); 

                numeric.int16   = 
                numeric.int16u  = 
                string.char16   = &bits16;

                numeric.int32   = 
                numeric.int32u  = 
                numeric.float32 = 
                string.char32   = &bits32;

                numeric.int64   = 
                numeric.int64u  = 
                numeric.float64 = &bits64;

                numeric.float80 = &bits80;
        }

        /***********************************************************************
        
        ***********************************************************************/
        
        override char[] toString()
        {
                return "endian reader";
        }

        /***********************************************************************
        
        ***********************************************************************/

        final void bits16 (void* dst, uint count)
        {               
                read (dst, count);         
                ByteSwap.swap16 (dst, count);      
        }

        /***********************************************************************
        
        ***********************************************************************/

        final void bits32 (void* dst, uint count)
        {      
                read (dst, count);   
                ByteSwap.swap32 (dst, count);      
        }

        /***********************************************************************
        
        ***********************************************************************/

        final void bits64 (void* dst, uint count)
        {      
                read (dst, count);   
                ByteSwap.swap64 (dst, count);      
        }

        /***********************************************************************
        
        ***********************************************************************/

        final void bits80 (void* dst, uint count)
        {      
                read (dst, count);   
                ByteSwap.swap80 (dst, count);      
        }
}
