/*******************************************************************************

        @file Writer.d
        

        Copyright (C) 2004 Kris Bell
        
        This software is provided 'as-is', without any express or implied
        warranty. In no event will the authors be held liable for damages
        of any kind arising from the use of this software.
        
        Permission is hereby granted to anyone to use this software for any 
        purpose, including commercial applications, and to alter it and/or 
        redistribute it freely, subject to the following restrictions:
        
        1. The origin of this software must not be misrepresented; you must 
           not claim that you wrote the original software. If you use this 
           software in a product, an acknowledgment within documentation of 
           said product would be appreciated but is not required.

        2. Altered source versions must be plainly marked as such, and must 
           not be misrepresented as being the original software.

        3. This notice may not be removed or altered from any distribution
           of the source.


                        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        @version        Initial version, March 2004      
        @author         Kris

*******************************************************************************/

module mango.io.Writer;

public import mango.io.AbstractWriter;

/*******************************************************************************

        Binary data Writer. Writers provide the means to append formatted 
        data to an IBuffer, and expose a convenient method of handling a
        variety of data types. In addition to writing native types such
        as integer and char[], writers also process any class which has
        implemented the IWritable interface (one method).

        See AbstractWriter for usage examples.

*******************************************************************************/

class Writer : AbstractWriter
{  
        typedef void function (void* dst, uint count) Mutator;

        /***********************************************************************
        
                Construct a Writer upon the provided IBuffer. All formatted
                ouput will be appended to this buffer.

        ***********************************************************************/

        this (IBuffer buffer)
        {
                super (buffer);

                numeric.int1    = 
                numeric.int8    = 
                numeric.int8u   =
                numeric.int16   = 
                numeric.int16u  = 
                numeric.int32   = 
                numeric.int32u  =
                numeric.int64   =
                numeric.int64u  =
                numeric.float32 =
                numeric.float64 =
                numeric.float80 =
                string.char8    = 
                string.char16   = 
                string.char32   = &write;
        }
     
        /***********************************************************************
        
                Construct a Writer on the buffer associated with the given
                conduit.

        ***********************************************************************/

        this (IConduit conduit)
        {
                this (conduit.createBuffer);
        }

        /***********************************************************************
        
        ***********************************************************************/
        
        override char[] toString()
        {
                return "binary writer";
        }

        /***********************************************************************
        
        ***********************************************************************/

        final void write (void* src, uint count)
        {
                static void mutate (void* dst, uint count)
                {
                }

                write (src, count, ~0, &mutate);
        }

        /***********************************************************************
        
        ***********************************************************************/

        final void write (void* src, uint count, uint mask, Mutator mutate)
        {
                while (count)
                      {
                      // write as much space as there is available in the buffer
                      uint available = buffer.writable() & mask;

                      // cap bytes written
                      if (available > count)
                          available = count;

                      // copy them over and adjust output
                      mutate (buffer.put(src, available), available);
                       
                      // bump counters; flush if we used all buffer space
                      src += available;
                      if (count -= available)
                          if (buffer.getConduit)
                              buffer.flush ();
                          else
                             if (! buffer.grow (count))
                                   throw ovf;
                      }
        }
}
